#version 150

uniform sampler2D DiffuseSampler;
uniform sampler2D DepthSampler;
uniform vec2 OutSize;

uniform float sunAngle;
uniform mat4 u_ProjMatrix;

uniform sampler2D TranslucentSampler;
uniform sampler2D TranslucentDepth;
uniform sampler2D ItemEntitySampler;
uniform sampler2D ItemEntityDepth;
uniform sampler2D ParticlesSampler;
uniform sampler2D ParticlesDepth;
uniform sampler2D WeatherSampler;
uniform sampler2D WeatherDepth;
uniform sampler2D CloudsSampler;
uniform sampler2D CloudsDepth;

in vec2 texCoord;
out vec4 fragColor;

float adjustValue(float value) {
    value = value * 3.5;
    value = (value - 0.5) * 4.0 + 0.5;
    return clamp(value, 0.0, 1.0);
}

float getDepthOffset(vec2 offset) {
    return texture(DepthSampler, texCoord + offset).r;
}

bool isTransparentElement(vec4 color, float depth) {
    bool isWaterLike = color.a < 0.9 && color.b > max(color.r, color.g);
    bool isParticleLike = (color.a < 0.95 && color.a > 0.1 && depth < 0.5);
    bool isGlassLike = color.a < 0.95 && color.a > 0.1 && abs(color.r - color.g) < 0.1 && abs(color.g - color.b) < 0.1;
    return isWaterLike || isParticleLike || isGlassLike;
}

// pos from depth
vec3 getViewPos(sampler2D depthTex, vec2 uv) {
    float z_ndc = texture(depthTex, uv).r * 2.0 - 1.0;
    vec2 ndc    = uv * 2.0 - 1.0;
    vec4 clip   = vec4(ndc, z_ndc, 1.0);
    vec4 view   = inverse(u_ProjMatrix) * clip;  // you must pass your projection matrix
    return view.xyz / view.w;
}

// sampling right and up neighbors
vec3 computeNormal(sampler2D depthTex, vec2 uv, vec2 texelSize) {
    vec3 p  = getViewPos(depthTex, uv);
    vec3 px = getViewPos(depthTex, uv + vec2(texelSize.x, 0.0));
    vec3 py = getViewPos(depthTex, uv + vec2(0.0, texelSize.y));
    return normalize(cross(px - p, py - p));
}


void main() {
    vec4 originalColor = texture(CloudsSampler, texCoord);
    originalColor = texture(DiffuseSampler, texCoord);
    /*originalColor += texture(ParticlesSampler, texCoord);
    originalColor += texture(WeatherSampler, texCoord);
    originalColor += texture(ItemEntitySampler, texCoord);
    originalColor += texture(TranslucentSampler, texCoord);*/
    
    float mainDepth = texture(DepthSampler, texCoord).r;
    /*mainDepth += texture(ItemEntityDepth, texCoord).r;
    mainDepth += texture(CloudsDepth, texCoord).r;
    mainDepth += texture(ParticlesDepth, texCoord).r);
    mainDepth += texture(WeatherDepth, texCoord).r;
    mainDepth += texture(TranslucentDepth, texCoord).r;*/

    float depth = mainDepth;
    
    vec2 texelSize = 1.0 / OutSize;
    
    // sampling neighbours
    float depthL = getDepthOffset(vec2(-texelSize.x * 2.0, 0.0));
    float depthR = getDepthOffset(vec2(texelSize.x * 2.0, 0.0));
    float depthU = getDepthOffset(vec2(0.0, -texelSize.y * 2.0));
    float depthD = getDepthOffset(vec2(0.0, texelSize.y * 2.0));
    
    // edge detection
    float diffX = abs(depthL - depthR);
    float diffY = abs(depthU - depthD);
    float edge = max(diffX, diffY);
    
    float shiftedDepthL = getDepthOffset(vec2(-texelSize.x * 4.0, 0.0));
    float shiftedDepthR = getDepthOffset(vec2(texelSize.x * 4.0, 0.0));
    float shiftedDepthU = getDepthOffset(vec2(0.0, -texelSize.y * 4.0));
    float shiftedDepthD = getDepthOffset(vec2(0.0, texelSize.y * 4.0));
    
    // shifted edge detection
    float shiftedDiffX = abs(shiftedDepthL - shiftedDepthR);
    float shiftedDiffY = abs(shiftedDepthU - shiftedDepthD);
    float shiftedEdge = max(shiftedDiffX, shiftedDiffY);
    
    // adaptive edging >:]
    float closeObjectFactor = smoothstep(0.0, 0.3, depth);
    float finalEdge = max(0.0, edge - shiftedEdge * closeObjectFactor * 0.7);
    float outline = finalEdge > 0.000001 ? 1.0 : 0.0;
    
    vec4 result = originalColor;
    bool isTransparent = isTransparentElement(result, depth);
    
    if (isTransparent) {
        float shimmer = sin(texCoord.x * 50.0 + texCoord.y * 30.0 + depth * 20.0) * 0.0 + 0.5;
        result.rgb += vec3(shimmer * 0.1);
        result.rgb += outline * vec3(0.1, 0.1, 0.1);
    } else {
        result += result * outline * 0.35 + vec4(outline, outline, outline, 0.0) * 0.005;
    }

    vec4 itemColor = texture(ItemEntitySampler, texCoord);
    float itemDepth = texture(ItemEntityDepth, texCoord).r;
    if (itemColor.a > 0.0 && itemDepth < depth) {
        itemColor += itemColor * outline * 0.05 + vec4(outline, outline, outline, 0.0) * 0.05;
        result = mix(result, itemColor, itemColor.a);
        depth = itemDepth;
    }

    vec4 translucentColor = texture(TranslucentSampler, texCoord);
    float translucentDepth = texture(TranslucentDepth, texCoord).r;
    if (translucentColor.a > 0.0 && translucentDepth < depth) {
        if (isTransparentElement(translucentColor, translucentDepth)) {
            translucentColor.rgb += 0.2 * outline * vec3(0.1, 0.1, 0.1);
        }

        // Composite as before, but maybe scale alpha if you like:
        result = mix(result, translucentColor, translucentColor.a);
        depth  = translucentDepth;
    }

    vec4 particleColor = texture(ParticlesSampler, texCoord);
    float particleDepth = texture(ParticlesDepth, texCoord).r;
    if (particleColor.a > 0.0 && particleDepth < depth) {
        if (isTransparentElement(particleColor, particleDepth)) {
            float shimmer = sin(texCoord.x * 50.0 + texCoord.y * 30.0 + particleDepth * 20.0) * 0.5 + 0.5;
            particleColor.rgb += vec3(shimmer * 0.1);
            particleColor.rgb += outline * vec3(0.2, 0.3, 0.4);
        }
        result = mix(result, particleColor, particleColor.a);
        depth = particleDepth;
    }

    vec4 weatherColor = texture(WeatherSampler, texCoord);
    float weatherDepth = texture(WeatherDepth, texCoord).r;
    if (weatherColor.a > 0.0 && weatherDepth < depth) {
        if (isTransparentElement(weatherColor, weatherDepth)) {
            float shimmer = sin(texCoord.x * 50.0 + texCoord.y * 30.0 + weatherDepth * 20.0) * 0.5 + 0.5;
            weatherColor.rgb += vec3(shimmer * 0.1);
            weatherColor.rgb += outline * vec3(0.2, 0.3, 0.4);
        }
        result = mix(result, weatherColor, weatherColor.a);
        depth = weatherDepth;
    }

    vec4 cloudColor = texture(CloudsSampler, texCoord);
    float cloudDepth = texture(CloudsDepth, texCoord).r;
    if (cloudColor.a > 0.0 && cloudDepth < mainDepth) {
        if (isTransparentElement(cloudColor, cloudDepth)) {
            float shimmer = sin(texCoord.x * 50.0 + texCoord.y * 30.0 + cloudDepth * 20.0) * 0.0 + 1.5;
            //float shimmer = texCoord.x * 50.0 + texCoord.y * 30.0 + cloudDepth * 20.0;
            cloudColor.rgb += cloudColor.rgb + (cloudColor.rgb + vec3(shimmer));
            //cloudColor.rgb *= pow(vec3(cloudColor.r), vec3(2.5));
            cloudColor.rgb += depth * 1.0;
            cloudColor.rgb += outline * vec3(0.2, 0.2, 0.2);
        }
        //result = mix(cloudColor, result, cloudColor.a * 0.5);
        //cloudColor = mix(result, cloudColor, cloudColor.a * 0.5);
        //result += cloudColor;
        //result = mix(cloudColor, result, cloudColor.a * 0.5);
        //result = result;
        //depth = cloudDepth;
    }

    fragColor = vec4(clamp(result.rgb, 0.0, 1.0), result.a);
}
