#version 150

uniform sampler2D DiffuseSampler;
uniform sampler2D DepthSampler;
uniform vec2 OutSize;

uniform sampler2D TranslucentSampler;
uniform sampler2D TranslucentDepth;
uniform sampler2D ItemSampler;
uniform sampler2D ItemDepth;
uniform sampler2D ParticleSampler;
uniform sampler2D ParticleDepth;

in vec2 texCoord;
out vec4 fragColor;

float adjustValue(float value) {
    value = value * 3.5;
    value = (value - 0.5) * 4.0 + 0.5;
    return clamp(value, 0.0, 1.0);
}

float getDepthOffset(vec2 offset) {
    return texture(DepthSampler, texCoord + offset).r;
}

// Detect if pixel is likely water or particle
bool isTransparentElement(vec4 color, float depth) {
    bool isWaterLike = color.a < 0.9 && color.b > max(color.r, color.g);
    bool isParticleLike = (color.a < 0.95 && color.a > 0.1 && depth < 0.5);
    return isWaterLike || isParticleLike;
}

void main() {
    vec4 originalColor = texture(DiffuseSampler, texCoord);
    
    vec2 texelSize = 1.0 / OutSize;
    float depth = texture(DepthSampler, texCoord).r;
    
    // sampling neighbours
    float depthL = getDepthOffset(vec2(-texelSize.x * 2.0, 0.0));
    float depthR = getDepthOffset(vec2(texelSize.x * 2.0, 0.0));
    float depthU = getDepthOffset(vec2(0.0, -texelSize.y * 2.0));
    float depthD = getDepthOffset(vec2(0.0, texelSize.y * 2.0));
    
    // edge detection
    float diffX = abs(depthL - depthR);
    float diffY = abs(depthU - depthD);
    float edge = max(diffX, diffY);
    
    float shiftedDepthL = getDepthOffset(vec2(-texelSize.x * 4.0, 0.0));
    float shiftedDepthR = getDepthOffset(vec2(texelSize.x * 4.0, 0.0));
    float shiftedDepthU = getDepthOffset(vec2(0.0, -texelSize.y * 4.0));
    float shiftedDepthD = getDepthOffset(vec2(0.0, texelSize.y * 4.0));
    
    // shifted
    float shiftedDiffX = abs(shiftedDepthL - shiftedDepthR);
    float shiftedDiffY = abs(shiftedDepthU - shiftedDepthD);
    float shiftedEdge = max(shiftedDiffX, shiftedDiffY);
    
    // adaptive "edging"
    float closeObjectFactor = smoothstep(0.0, 0.3, depth);
    float finalEdge = max(0.0, edge - shiftedEdge * closeObjectFactor * 0.7);
    
    // threshold
    float threshold = 0.000001;
    
    float outline = finalEdge > threshold ? 1.0 : 0.0;
    
    bool isTransparent = isTransparentElement(originalColor, depth);
    
    vec4 finalColor;
    if (isTransparent) {
        //
        float shimmer = sin(texCoord.x * 50.0 + texCoord.y * 30.0 + depth * 20.0) * 0.5 + 0.5;
        finalColor = originalColor;
        finalColor.rgb += vec3(shimmer * 0.1);
        finalColor.rgb += outline * vec3(0.2, 0.3, 0.4);
    } else {
        //
        finalColor = originalColor + originalColor * outline * 0.2 + vec4(outline, outline, outline, 0.0) * 0.05;
    }
    
    finalColor.a = originalColor.a;
    
    // --- START COMPOSITE ---
    // 1) Opaque main
    vec4 result = finalColor; // base color with outlines

    // 2) Helper to composite any extra layer with same outline
    #define COMPOSITE_LAYER(colS, depS, doShimmer)
    do {
        vec4 c = texture(colS, texCoord);
        float d = texture(depS, texCoord).r;
        if (c.a > 0.0 && d < depthMain) {
            vec4 mod = c;
            if (doShimmer && isTransparentElement(c, d)) {
                float shimmer =
                    sin(texCoord.x*50.0 + texCoord.y*30.0 + d*20.0)*0.5 + 0.5;
                mod.rgb += shimmer * 0.1;
                mod.rgb += outline * vec3(0.2,0.3,0.4);
            } else {
                // same outline as solids
                mod = baseOutline(c);
            }
            result += mod * c.a;
            depthMain = d;  // ensure further layers obey depth
        }
    } while(0)

    // 3) Composite in proper order:
    COMPOSITE_LAYER(TranslucentSampler, TranslucentDepth, true);  // water/glass
    COMPOSITE_LAYER(ItemSampler,        ItemDepth,        false); // items/entities
    COMPOSITE_LAYER(ParticleSampler,    ParticleDepth,    true);  // particles

    // 4) Output final color
    fragColor = vec4(clamp(finalColor.rgb, 0.0, 1.0), finalColor.a);
}